@@ -36,6 +36,7 @@ public class Video {
 	private Player player = null;
 	private VideoPosition position = null;
 	private SubtitleOverlay overlay = null;
+	private SubtitleTracker tracker = null;
 	
 	private bool isLoaded = false;
 	private bool playPauseToggleIsSilent = false; //Used to indicate whether toggling the button should not issue the toggled signal
@@ -43,6 +44,7 @@ public class Video {
 	/* Constant strings */
 	private const string videoSetSubtitleStartIconFilename = "video-set-subtitle-start-16x.png";
 	private const string videoSetSubtitleEndIconFilename = "video-set-subtitle-end-16x.png";
+	private const string videoSetSubtitleStartEndIconFilename = "video-set-subtitle-start-end-30x.png";
 
 	public Video () {
 		videoArea = Base.GetWidget(WidgetNames.VideoAreaHBox) as HBox;
@@ -52,6 +54,7 @@ public class Video {
 		
 		position = new VideoPosition(player);
 		overlay = new SubtitleOverlay();
+		tracker = new SubtitleTracker();
 
 		SetCustomIcons();
 		Base.InitFinished += OnBaseInitFinished;
@@ -66,6 +69,10 @@ public class Video {
 	public SubtitleOverlay Overlay {
 		get { return overlay; }
 	}
+		
+	public SubtitleTracker Tracker {
+		get { return tracker; }
+	}
 	
 	public bool IsLoaded {
 		get { return isLoaded; }
@@ -113,8 +120,10 @@ public class Video {
 
 		player.Close();
 		overlay.Close();
+		tracker.Close();
 		position.Disable();
 		
+		
 		/* Update the frame */
 		frame.Child.Hide();
 		frame.Child.Show();
@@ -129,6 +138,13 @@ public class Video {
 	public void Quit () {
 		player.Close();
 	}
+		
+	public void SetLoopSelectionPlayback (bool enabled){
+		if (enabled)
+			Base.Ui.Video.Position.Changed += OnVideoPositionChangedLoopPlayback;
+		else
+			Base.Ui.Video.Position.Changed -= OnVideoPositionChangedLoopPlayback;
+	}
 	
 	public void Rewind () {
 		player.Rewind(position.SeekIncrement);
@@ -168,6 +184,11 @@ public class Video {
     	Seek(time);
 	}
 	
+	public void SelectNearestSubtitle () {		
+		int indexToSelect = tracker.FindSubtitleNearPosition(position.CurrentTime);
+		Base.Ui.View.Selection.Select(indexToSelect, true, true);
+	}
+	
 	/* Private methods */
 
 	private void Play () {
@@ -188,6 +209,11 @@ public class Video {
 		pixbuf = new Gdk.Pixbuf(null, videoSetSubtitleEndIconFilename);
 		image = Base.GetWidget(WidgetNames.VideoSetSubtitleEndButtonImage) as Image;
 		image.Pixbuf = pixbuf;
+			
+		/* Set the icon for the SetSubtitleStartEnd button */
+		pixbuf = new Gdk.Pixbuf(null, videoSetSubtitleStartEndIconFilename);
+		image = Base.GetWidget(WidgetNames.VideoSetSubtitleStartEndButtonImage) as Image;
+		image.Pixbuf = pixbuf;
 	}
 	
 	private void InitializeVideoFrame () {
@@ -229,6 +255,7 @@ public class Video {
 	private void SetSelectionDependentControlsSensitivity (bool sensitivity) {
 		Base.GetWidget(WidgetNames.VideoSetSubtitleStartButton).Sensitive = sensitivity;
 		Base.GetWidget(WidgetNames.VideoSetSubtitleEndButton).Sensitive = sensitivity;
+		Base.GetWidget(WidgetNames.VideoSetSubtitleStartEndButton).Sensitive = sensitivity;
 	}
 	
 	private void SilentDisablePlayPauseButton () {
@@ -258,7 +285,7 @@ public class Video {
 		ToggleButton button = Base.GetWidget(WidgetNames.VideoPlayPauseButton) as ToggleButton;
 		button.Toggled += OnPlayPauseButtonToggled;
 		
-		Base.Ui.View.Selection.Changed += OnSubtitleSelectionChanged;
+		Base.Ui.View.Selection.Changed += OnSubtitleSelectionChanged;		
 	}
 	
 	private void OnPlayPauseButtonToggled (object o, EventArgs args) {
@@ -302,10 +329,25 @@ public class Video {
 
 	private void OnSubtitleSelectionChanged (TreePath[] paths, Subtitle subtitle) {
 		if ((subtitle != null) && IsLoaded)
-			SetSelectionDependentControlsSensitivity(true);
+			SetSelectionDependentControlsSensitivity(true);			
 		else
 			SetSelectionDependentControlsSensitivity(false);
 	}
+	
+	/// <summary>Do loop playback when it's enabled, seeking to current selection on video position change.</summary>
+	private void OnVideoPositionChangedLoopPlayback (TimeSpan position) {
+		if (!(Base.IsDocumentLoaded))
+			return;
+
+		Subtitle firstSubtitle = Core.Base.Ui.View.Selection.FirstSubtitle;
+		if (firstSubtitle == null)
+			return;
+
+		Subtitle lastSubtitle = Core.Base.Ui.View.Selection.LastSubtitle;
+
+		if ((position < firstSubtitle.Times.Start) || (position > lastSubtitle.Times.End))
+			SeekToSelection();
+	}
 
 }
 
