@@ -37,11 +37,12 @@ public class SubtitleSelection {
 	public SubtitleSelection (TreeView tree) {
 		this.tree = tree;
 		this.selection = tree.Selection;
-		
+
 		selection.Mode = SelectionMode.Multiple;
-		
+
 		Base.InitFinished += OnBaseInitFinished;
 	}
+
 	
 	/* Events */
 	
@@ -149,6 +150,17 @@ public class SubtitleSelection {
     			return null;
     	}
     }
+		
+	/// <summary>The last selected subtitle. If there is only one selected, the first is returned.</summary>
+    public Subtitle LastSubtitle {
+    	get {
+    		TreePath path = this.LastPath;
+    		if (path != null)
+    			return Base.Document.Subtitles[path];
+    		else
+    			return null;
+    	}
+    }
 
     /// <summary>The number of selected paths.</summary>
     public int Count {
@@ -213,18 +225,39 @@ public class SubtitleSelection {
     /// <param name="path">The path to select. If it's null, all paths will be unselected.</param>
     /// <param name="align">Whether to align the selected path to the center if the path isn't visible and scrolling is needed.</param>
     /// <param name="reselect">Whether to reselect the path if it's already the only selected path.</param>
+    /// <param name="checkAllPaths">If multiple paths are selected and reselect isn't required, check if the current path is contained in the selection.
+    /// If the subtitle is within the selection but we want to guarantee that only that single subtitle is selected, set this to false.</param>
     /// <remarks>The input focus will be placed on the path.</remarks>
-    public void Select (TreePath path, bool align, bool reselect) {
+    public void Select (TreePath path, bool align, bool reselect, bool checkAllPaths) {
    		if (path == null) {
    			UnselectAll();
    			return;
    		}
-   		if ((!reselect) && (Count == 1) && (selection.PathIsSelected(path))) //No reselection is required and path is already the only selected path
+   		
+   		if ((!reselect) && (checkAllPaths || (Count == 1)) && (selection.PathIsSelected(path))) //No reselection is required and path is already the only selected path
    			return;
 
 		SetFocus(path, align);
 	}
 	
+	public void Select (TreePath path, bool align, bool reselect) {
+		Select(path, align, reselect, false);
+	}
+	
+	/// <summary>Selects the specified index, possibly aligning it to the center and/or reselecting it.</summary>
+	/// <param name="index">The index of the subtitle to select.</param>
+    /// <param name="align">Whether to align the selected path to the center if the path isn't visible and scrolling is needed.</param>
+    /// <param name="reselect">Whether to reselect the path if it's already the only selected path.</param>
+    /// <remarks>The subtitle is only selected if it exists. The input focus will be placed on the path.</remarks>
+    public void Select (int index, bool align, bool reselect, bool checkAllPaths) {
+    	if ((index >= 0) && (index < Base.Document.Subtitles.Count))
+    		Select(Util.IntToPath(index), align, reselect, checkAllPaths);
+	}
+	
+	public void Select (int index, bool align, bool reselect) {
+		Select(index, align, reselect, false);
+	}
+	
 	/// <summary>Selects a <see cref="TreePath" />, activates it and selects text in the subtitle it refers to.</summary>
 	/// <param name="path">The path to select. If it's null, all paths will be unselected.</param>
     /// <param name="align">Whether to align the selected path to the center if the path isn't visible and scrolling is needed.</param>
@@ -240,7 +273,7 @@ public class SubtitleSelection {
 		Select(path, align, reselect);
 		Core.Base.Ui.Edit.TextFocusOnSelection(start, end, textType);		
 	}
-    
+		
     /// <summary>Selects the first subtitle.</summary>
     /// <remarks>The subtitle is only selected if it exists.</remarks>
     public void SelectFirst () {
@@ -337,7 +370,7 @@ public class SubtitleSelection {
    	/// <param name="align">Whether to align the path to the center if it isn't visible.</param>
    	/// <remarks>The path is selected and the input </remarks>
    	private void SetFocus (TreePath path, bool align) {
-		ScrollToFocus(path, true);
+		ScrollToFocus(path, align);
    		tree.SetCursor(path, null, false);
    	}
 	
